package com.hisschemoller.utils.delegates 
{
	import de.popforge.audio.output.Sample;
	import de.popforge.format.wav.WavFormat;
	
	import com.hisschemoller.utils.delegates.vo.AudioLoaderVO;
	
	import org.audiofx.mp3.MP3FileReferenceLoader;
	import org.audiofx.mp3.MP3SoundEvent;
	import org.puremvc.as3.interfaces.IFacade;
	
	import mx.rpc.Responder;
	
	import flash.events.Event;
	import flash.utils.ByteArray;	
	/**
	 * @author Wouter Hisschemoller
	 * (c) Feb 26, 2010
	 */
	public class AudioDecoderDelegate 
	{
		private var _responder : Responder;
		private var _facade : IFacade;
		private var _loadedStack : Vector.<AudioLoaderVO>;
		private var _decodingStack : Vector.<AudioLoaderVO> = new Vector.<AudioLoaderVO>();
		private var _decodedStack : Vector.<AudioLoaderVO> = new Vector.<AudioLoaderVO>();
		
		public function AudioDecoderDelegate(responder : Responder, facade : IFacade)
		{
			_responder = responder;
			_facade = facade;
		}
		
		/**
		 * 
		 */
		public function decodeAudio(loadedStack : Vector.<AudioLoaderVO>) : void
		{
			_loadedStack = loadedStack;
			
			decodeAudioFile();
		}

		/**
		 * All audio loading is done, now decode the byteArrays.
		 */
		private function decodeAudioFile() : void
		{
			if(_loadedStack.length == 0 && _decodingStack.length == 0)
			{
				/** All done. */
				_responder.result(_decodedStack);
				return;
			}
			
			/** Get next object to decode. */
			var audioLoaderVO : AudioLoaderVO = _loadedStack.shift();
			
			/** Store object in loading list. */
			_decodingStack.push(audioLoaderVO);
			
			switch(audioLoaderVO.fileExtention)
			{
				case "mp3":
					var fileReference : MP3FileReferenceLoader = new MP3FileReferenceLoader();
					fileReference.addEventListener(Event.COMPLETE, mp3DecodeCompleteHandler);
					fileReference.parseMP3ByteArray(audioLoaderVO.byteArray, audioLoaderVO.url);
					break;
					
				case "wav":
					var wavFormat : WavFormat = WavFormat.decode(audioLoaderVO.byteArray);
					/** Rewrite the WAV byteArray. */
					var byteArray : ByteArray = new ByteArray();
					var n : int = wavFormat.numSamples;
					for(var i : uint = 0;i < n; i++)
					{
						var sample : Sample = Sample(wavFormat.samples[i]);
						byteArray.writeFloat(sample.left);
						byteArray.writeFloat(sample.right);
					}
					audioLoaderVO.byteArray = byteArray;
					audioLoaderVO.channels = wavFormat.channels;
					audioLoaderVO.numSamplesTotal = wavFormat.numSamples;
					_decodedStack.push(_decodingStack.splice(_decodingStack.indexOf(audioLoaderVO), 1)[0]);
					decodeAudioFile();
					break;
			}
		}

		/**
		 * Decoding of an MP3 is done.
		 * @param event : MP3SoundEvent width Sound object as property sound.
		 */
		private function mp3DecodeCompleteHandler(event : MP3SoundEvent) : void
		{
			MP3FileReferenceLoader(event.target).removeEventListener(Event.COMPLETE, mp3DecodeCompleteHandler);
			
			var audioLoaderVO : AudioLoaderVO = getAudioDataByURL(event.id, _decodingStack);
			audioLoaderVO.byteArray = new ByteArray();
			audioLoaderVO.channels = event.channels;
			audioLoaderVO.numSamplesTotal = event.sound.extract(audioLoaderVO.byteArray, (44100 * 60 * 60), 0);
			
			_decodedStack.push(_decodingStack.splice(_decodingStack.indexOf(audioLoaderVO), 1)[0]);
			decodeAudioFile();
		}

		/**
		 * Get sound data for specified sound
		 */
		private function getAudioDataByURL(url : String, stack : Vector.<AudioLoaderVO>) : AudioLoaderVO 
		{
			for each (var audioLoadData : AudioLoaderVO in stack)
			{
				if(audioLoadData.url == url) return audioLoadData;
			}

			return null;
		}
	}
}
