package com.hisschemoller.utils.delegates 
{
	import com.hisschemoller.model.vo.AlertVO;
	
	import mx.rpc.Responder;
	
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.events.SecurityErrorEvent;
	import flash.net.FileFilter;
	import flash.net.FileReference;	
	/**
	 * @author Wouter Hisschemoller
	 * (c) Feb 26, 2010
	 */
	public class AudioOpenDelegate 
	{
		private var _responder : Responder;
		private var _fileReference : FileReference;
		
		public function AudioOpenDelegate(responder : Responder)
		{
			_responder = responder;
		}
		
		/**
		 * 
		 */
		public function openAudioFile() : void
		{
			_fileReference = new FileReference();
			_fileReference.addEventListener(Event.SELECT, fileReferenceSelectHandler, false, 0, true);
			_fileReference.addEventListener(Event.CANCEL, noSelectionHandler, false, 0, true);
			_fileReference.addEventListener(IOErrorEvent.IO_ERROR, selectionErrorHandler, false, 0, true);
			_fileReference.addEventListener(SecurityErrorEvent.SECURITY_ERROR, securityErrorHandler);
			_fileReference.browse([ new FileFilter("MP3 and WAV Files", "*.mp3; *.wav") ]);
		}
		
		/**
		 * 
		 */
		private function noSelectionHandler(inEvent : Event) : void
		{
			removeListeners();
		}
		
		/**
		 * 
		 */
		private function selectionErrorHandler(inEvent : Event) : void
		{
			removeListeners();
			_responder.fault(new AlertVO("Error", "An error occurred while selecting a file. Please try again."));
		}

		/**
		 * @param event : Input / output error event.
		 */
		private function securityErrorHandler(event : SecurityErrorEvent) : void
		{
			_responder.fault(new AlertVO("Error opening file", "An security error occurred while selecting a file. Please try again."));
		}

		/**
		 * A local file is selected, convert the FileReference.data byteArray to a Sound object
		 */
		private function fileReferenceSelectHandler(event : Event) : void
		{
			removeListeners();
			
			switch(_fileReference.name.substr(_fileReference.name.length - 4, 4).toLowerCase())
			{
				case ".mp3":
				case ".wav":
					_fileReference.addEventListener(Event.COMPLETE, fileReferenceCompleteHandler, false, 0, true);
					_fileReference.load();
					break;
				default:
					_responder.fault(new AlertVO("Error opening file", "The selected file could not be opened. Only WAV and MP3 files are supported. Please try again."));
					break;
			}
		}

		/**
		 * A WAV file load from the local computer is complete.
		 * @param event : Event.
		 */
		private function fileReferenceCompleteHandler(event : Event) : void
		{
			removeListeners();
			
			_responder.result(_fileReference);
		}

		/**
		 * 
		 */
		private function removeListeners() : void
		{
			_fileReference.removeEventListener(Event.SELECT, fileReferenceSelectHandler);
			_fileReference.removeEventListener(Event.CANCEL, noSelectionHandler);
			_fileReference.removeEventListener(IOErrorEvent.IO_ERROR, selectionErrorHandler);
			_fileReference.removeEventListener(SecurityErrorEvent.SECURITY_ERROR, securityErrorHandler);
			_fileReference.removeEventListener(Event.COMPLETE, fileReferenceCompleteHandler);
		}
	}
}
