package com.hisschemoller.utils.parameter 
{
	/**
	 * @author Wouter Hisschemoller
	 * (c) Mar 11, 2010
	 * 
	 * From code by James McCartney at musicdsp:
	 * http://musicdsp.org/showArchiveComment.php?ArchiveID=94
	 * 
	 * Notes :
	 * These two functions reproduce a traditional professional mixer fader taper.

	 * MMtoDB converts millimeters of fader travel from the
	 * bottom of the fader for a 100 millimeter fader into
	 * decibels. DBtoMM is the inverse.

	 * The taper is as follows from the top:
	 * The top of the fader is +10 dB
	 * 100 mm to 52 mm : -5 dB per 12 mm
	 * 52 mm to 16 mm : -10 dB per 12 mm
	 * 16 mm to 4 mm : -20 dB per 12 mm
	 * 4 mm to 0 mm : fade to zero. (in these functions I go to -200dB
	 * which is effectively zero for up to 32 bit audio.)
	 */
	import de.popforge.parameter.IMapping;		
	public class MappingNumberFaderTaper implements IMapping
	{
		private var _min : Number;
		private var _max : Number;

		public function MappingNumberFaderTaper(min : Number = 0, max : Number = 1)
		{
			_min = min;
			_max = max;
		}
		
		/**
		 * @param normalizedValue: value between 0 and 1.
		 */
		public function map(normalizedValue : Number) : *
		{
			normalizedValue = (1 - normalizedValue) * 100;

			if(normalizedValue <= 0.0) var db : Number = 10.0;
			else if(normalizedValue < 48.0) db = 10.0 - 5.0 / 12.0 * normalizedValue;
			else if(normalizedValue < 84.0) db = -10.0 - 10.0 / 12.0 * (normalizedValue - 48.0);
			else if(normalizedValue < 96.0) db = -40.0 - 20.0 / 12.0 * (normalizedValue - 84.0);
			else if(normalizedValue < 100.0) db = -60.0 - 35.0 * (normalizedValue - 96.0);
			else db = -200.0;
			
			return db;
		}
		
		/**
		 * Maps value to normalized value between 0 and 1.
		 * @return Number: decibel from -200 to 10?
		 */
		public function mapInverse(decibel : *) : Number
		{
			if(decibel >= 10.0) var normalizedValue : Number = 0.0;
			else if (decibel > -10.0) normalizedValue = -12.0 / 5.0 * (decibel - 10.0);
			else if (decibel > -40.0) normalizedValue = 48.0 - 12.0 / 10.0 * (decibel + 10.0);
			else if (decibel > -60.) normalizedValue = 84.0 - 12.0 / 20.0 * (decibel + 40.0);
			else if (decibel > -200.) normalizedValue = 96.0 - 1.0/35.0 * (decibel + 60.0);
			else normalizedValue = 100.0;
			
			return (100.0 - normalizedValue) / 100.0;
		}
	}
}
