package com.hisschemoller.view 
{
	import com.hisschemoller.AppFacade;
	import com.hisschemoller.model.AudioDriverProxy;
	import com.hisschemoller.model.WaveDisplayProxy;
	import com.hisschemoller.model.vo.AudioStreamInfoVO;
	import com.hisschemoller.model.vo.WaveDisplayInfoVO;
	import com.hisschemoller.utils.audio.processors.IAudioProcessor;
	import com.hisschemoller.view.components.WaveDisplay;

	import org.puremvc.as3.interfaces.IMediator;
	import org.puremvc.as3.interfaces.INotification;
	import org.puremvc.as3.patterns.mediator.Mediator;

	import flash.events.Event;
	import flash.events.MouseEvent;

	/**
	 * (c) Copyright LBi Lost Boys 2009
	 * @author wouter.hisschemoller
	 */
	public class WaveDisplayMediator extends Mediator implements IMediator 
	{
		private var _displayProxy : WaveDisplayProxy;

		public function WaveDisplayMediator(name : String, viewComponent : Object)
		{
			super(name, viewComponent);
		}

		/**
		 * 
		 */
		override public function listNotificationInterests() : Array
		{
			return [AudioDriverProxy.ENTER_FRAME,
					AppFacade.PROCESSOR_ADDED,
					AppFacade.PROCESSOR_REMOVED,
					WaveDisplayProxy.GRAPHIC_UPDATED,
					WaveDisplayProxy.HORIZONTAL_ZOOM_CHANGED,
					WaveDisplayProxy.VERTICAL_ZOOM_CHANGED,
					WaveDisplayProxy.CHANNELS_CHANGED];
		}

		/**
		 * 
		 */
		override public function handleNotification(notification : INotification) : void
		{
			switch(notification.getName())
			{
				case AudioDriverProxy.ENTER_FRAME:
					waveDisplay.enterFrame(AudioStreamInfoVO(notification.getBody()));
					break;
				case AppFacade.PROCESSOR_ADDED:
					if(getMediatorName().indexOf(notification.getType()) != -1) waveDisplay.processorAdded(IAudioProcessor(notification.getBody()));
					break;
				case AppFacade.PROCESSOR_REMOVED:
					if(getMediatorName().indexOf(notification.getType()) != -1) waveDisplay.processorRemoved();
					break;
				case WaveDisplayProxy.GRAPHIC_UPDATED:
					var displayVO : WaveDisplayInfoVO = WaveDisplayInfoVO(notification.getBody());
					waveDisplay.updateGraphic(displayVO.bitmapData);
					waveDisplay.updatePositionScrollBar(displayVO.numSamplesInView, displayVO.numSamplesTotal, displayVO.firstSampleInView, displayVO.currentSamplesPerPixel);
					break;
				case WaveDisplayProxy.HORIZONTAL_ZOOM_CHANGED:
					waveDisplay.updateHZoomSlider(_displayProxy.horizontalZoomLevel);
					break;
				case WaveDisplayProxy.VERTICAL_ZOOM_CHANGED:
					waveDisplay.updateVZoomSlider(Number(notification.getBody()));
					break;
				case WaveDisplayProxy.CHANNELS_CHANGED:
					waveDisplay.updateChannels(uint(notification.getBody()));
					break;
			}
		}

		/**
		 * 
		 */
		override public function onRegister() : void
		{
			waveDisplay.addEventListener(WaveDisplay.HORIZONTAL_ZOOM_CHANGE, handleHZoomSliderChange);
			waveDisplay.addEventListener(WaveDisplay.VERTICAL_ZOOM_CHANGE, handleVZoomSliderChange);
			waveDisplay.addEventListener(WaveDisplay.POSITION_CHANGE, handlePositionSliderChange);
			waveDisplay.addEventListener(WaveDisplay.MOUSE_DOWN, handleGraphicMouseDown);
			waveDisplay.addEventListener(WaveDisplay.MOUSE_DRAG, handleGraphicMouseDrag);
			waveDisplay.addEventListener(MouseEvent.DOUBLE_CLICK, graphicDoubleClickHandler);
			
			_displayProxy = facade.retrieveProxy(getMediatorName()) as WaveDisplayProxy;
			_displayProxy.mediatorRegistered();
		}

		/**
		 * 
		 */
		private function handleHZoomSliderChange(event : Event) : void
		{
			_displayProxy.horizontalZoomLevel = waveDisplay.hZoomSliderValue;
		}

		/**
		 * 
		 */
		private function handleVZoomSliderChange(event : Event) : void
		{
			_displayProxy.verticalZoomLevel = waveDisplay.vZoomSliderValue;
		}

		/**
		 * 
		 */
		private function handlePositionSliderChange(event : Event) : void
		{
			_displayProxy.firstSampleInView = waveDisplay.positionScrollBarPosition;
		}

		/**
		 * Mouse is pressed on the wave display area.
		 */
		private function handleGraphicMouseDown(event : Event) : void
		{
			_displayProxy.initializeDrag(waveDisplay.graphicMouseX, waveDisplay.graphicMouseY);
		}

		/**
		 * Mouse is dragged on the wave display area.
		 */
		private function handleGraphicMouseDrag(event : Event) : void
		{
			_displayProxy.updateDrag(waveDisplay.graphicMouseX, waveDisplay.graphicMouseY);
		}

		/**
		 * Mouse is dragged on the wave display area.
		 */
		private function graphicDoubleClickHandler(event : MouseEvent) : void
		{
			_displayProxy.horizontalZoomLevel = 1;
		}

		/**
		 * 
		 */
		private function get waveDisplay() : WaveDisplay
		{
			return viewComponent as WaveDisplay;
		}
	}
}
