package com.hisschemoller.view 
{
	import com.hisschemoller.AppFacade;
	import com.hisschemoller.model.WaveDisplayProxy;
	import com.hisschemoller.model.WaveEditorProxy;
	import com.hisschemoller.model.vo.WaveDisplayInfoVO;
	import com.hisschemoller.view.components.WaveEditor;

	import org.puremvc.as3.interfaces.IMediator;
	import org.puremvc.as3.interfaces.INotification;
	import org.puremvc.as3.patterns.mediator.Mediator;

	import flash.events.Event;

	/**
	 * (c) Copyright LBi Lost Boys 2009
	 * @author wouter.hisschemoller
	 */
	public class WaveEditorMediator extends Mediator implements IMediator 
	{
		private var _editorProxy : WaveEditorProxy;
		private var _displayProxy : WaveDisplayProxy;

		public function WaveEditorMediator(name : String, viewComponent : Object)
		{
			super(name, viewComponent);
		}

		/**
		 * 
		 */
		override public function listNotificationInterests() : Array
		{
			return [WaveDisplayProxy.GRAPHIC_UPDATED,
					AppFacade.PROCESSOR_ADDED,
					AppFacade.PROCESSOR_REMOVED,
					WaveEditorProxy.PROGRAM_START_CHANGED,
					WaveEditorProxy.PROGRAM_END_CHANGED,
					WaveEditorProxy.PROGRAM_TRANSPOSE_CHANGED,
					WaveEditorProxy.PROGRAM_TUNE_CHANGED,
					WaveEditorProxy.PROGRAM_REVERSE_CHANGED ];
		}

		/**
		 * 
		 */
		override public function handleNotification(notification : INotification) : void
		{
			switch(notification.getName())
			{
				case WaveDisplayProxy.GRAPHIC_UPDATED:
					var waveDisplayVO : WaveDisplayInfoVO = WaveDisplayInfoVO(notification.getBody());
					waveEditor.graphicsUpdated(waveDisplayVO.firstSampleInView, waveDisplayVO.numSamplesInView);
					break;
				case AppFacade.PROCESSOR_ADDED:
					if(notification.getType() == getMediatorName()) waveEditor.updatePlaybackState(true);
					break;
				case AppFacade.PROCESSOR_REMOVED:
					if(notification.getType() == getMediatorName()) waveEditor.updatePlaybackState(false);
					break;
				case WaveEditorProxy.PROGRAM_START_CHANGED:
					waveEditor.startLocatorChanged(int(notification.getBody()));
					break;
				case WaveEditorProxy.PROGRAM_END_CHANGED:
					waveEditor.endLocatorChanged(int(notification.getBody()));
					break;
				case WaveEditorProxy.PROGRAM_TRANSPOSE_CHANGED:
					waveEditor.transpose = int(notification.getBody());
					break;
				case WaveEditorProxy.PROGRAM_TUNE_CHANGED:
					waveEditor.tune = Number(notification.getBody());
					break;
				case WaveEditorProxy.PROGRAM_REVERSE_CHANGED:
					waveEditor.reverse = Boolean(notification.getBody());
					break;
			}
		}

		/**
		 * 
		 */
		override public function onRegister() : void
		{
			_editorProxy = facade.retrieveProxy(getMediatorName()) as WaveEditorProxy;
			_displayProxy = facade.retrieveProxy(getMediatorName() + WaveDisplayProxy.NAME_SUFFIX) as WaveDisplayProxy;
			
			waveEditor.addEventListener(WaveEditor.START_PLAY_LOCATOR_CHANGE, handleLocatorChange);
			waveEditor.addEventListener(WaveEditor.END_PLAY_LOCATOR_CHANGE, handleLocatorChange);
			waveEditor.addEventListener(WaveEditor.REVERSE_CHANGE, reverseChangeHandler);
			waveEditor.addEventListener(WaveEditor.TRANSPOSE_CHANGE, transposeChangeHandler);
			waveEditor.addEventListener(WaveEditor.TUNE_CHANGE, tuneChangeHandler);
			
			_editorProxy.mediatorRegistered();
		}

		/**
		 * Called when a locator is dragged
		 * @param event : event whose type indicates which locator was dragged.
		 */
		private function handleLocatorChange(event : Event) : void
		{
			var sampleIndex : int = _displayProxy.firstSampleInView + Math.round(_displayProxy.numSamplesInView * waveEditor.getLocatorValue(event.type));
			
			switch(event.type)
			{
				case WaveEditor.START_PLAY_LOCATOR_CHANGE:
					_editorProxy.programStart = sampleIndex;
					break;
				case WaveEditor.END_PLAY_LOCATOR_CHANGE:
					_editorProxy.programEnd = sampleIndex;
					break;
			}
		}

		/**
		 * Called when the reverse button is clicked.
		 */
		private function reverseChangeHandler(event : Event) : void
		{
			_editorProxy.programReverse = waveEditor.reverse;
		}

		/**
		 * Called when the transpose slider is moved.
		 */
		private function transposeChangeHandler(event : Event) : void
		{
			_editorProxy.programTranspose = waveEditor.transpose;
		}

		/**
		 * Called when the tune slider is moved.
		 */
		private function tuneChangeHandler(event : Event) : void
		{
			_editorProxy.programTune = waveEditor.tune;
		}

		/**
		 * 
		 */
		private function get waveEditor() : WaveEditor
		{
			return viewComponent as WaveEditor;
		}
	}
}
