package com.hisschemoller.view.components.ui 
{
	import com.bit101.components.ScrollBar;
	import com.bit101.components.Slider;

	import flash.display.DisplayObjectContainer;
	import flash.events.Event;
	import flash.events.MouseEvent;
	/**
	 * @author Wouter Hisschemoller
	 * (c) Mar 21, 2010
	 */
	public class BasicScrollBar extends ScrollBar 
	{
		protected var _slider : BasicScrollSlider;
		protected var _lineScrollSize : Number = 0; /** Amount that counts as scrolling one line (when a button is clicked). */
		protected var _pageScrollSize : Number = 0;

		public function BasicScrollBar(orientation : String, parent : DisplayObjectContainer = null, xpos : Number = 0, ypos : Number = 0, defaultHandler : Function = null)
		{
			super(orientation, parent, xpos, ypos, defaultHandler);
		}

		/**
		 * Creates and adds the child display objects of this component.
		 */
		override protected function addChildren() : void
		{
			_slider = new BasicScrollSlider(_orientation, this, 0, 0, onChange);
			_slider.addEventListener(BasicScrollSlider.PAGE_DOWN, pageChangeHandler);
			_slider.addEventListener(BasicScrollSlider.PAGE_UP, pageChangeHandler);
			
			_upButton = new TextButton(this, 0, 0, "", onUpClick);
			_downButton = new TextButton(this, 0, 0, "", onDownClick);
		}

		/**
		 * Convenience method to set the three main parameters in one shot.
		 * @param min The minimum value of the slider.
		 * @param max The maximum value of the slider.
		 * @param value The value of the slider.
		 */
		override public function setSliderParams(min : Number, max : Number, value : Number) : void
		{
			_slider.setSliderParams(min, max, value);
			
			if(_lineScrollSize == 0) _lineScrollSize = Math.max(1, Math.min(Math.abs(max - min) / 100), _slider.maximum);
			if(_pageScrollSize == 0) _pageScrollSize = Math.max(1, Math.min(Math.abs(max - min) / 10), _slider.maximum);
		}

		/**
		 * Sets the percentage of the size of the thumb button.
		 */
		override public function setThumbPercent(value : Number) : void
		{
			_slider.setThumbPercent(value);
		}
		
		/**
		 * The amount value changes to represent one page of content.
		 * Used when the slider back is clicked.
		 */
		public function setPageSize(pageScrollSize : Number) : void
		{
			_pageScrollSize = pageScrollSize;
		}

		/**
		 * Draws the visual ui of the component.
		 */
		override public function draw() : void
		{
			if(_orientation == Slider.VERTICAL)
			{
				_slider.x = 0;
				_slider.y = _width;
				_slider.width = _width;
				_slider.height = _height - (_width * 2);
				_upButton.setSize(_width, _width);
				_downButton.x = 0;
				_downButton.y = _height - _width;
				_downButton.setSize(_width, _width);
			}
			else
			{
				_slider.x = _height;
				_slider.y = 0;
				_slider.width = _width - (_height * 2);
				_slider.height = _height;
				_upButton.setSize(_height, _height);
				_downButton.x = _width - _height;
				_downButton.y = 0;
				_downButton.setSize(_height, _height);
			}
		}

		/**
		 * Sets / gets the current value of this scroll bar.
		 */
		override public function set value(value : Number) : void
		{
			if(!_slider.isDown) _slider.value = value;
		}

		override public function get value() : Number
		{
			return _slider.value;
		}
		
		/**
		 * 
		 */
		override protected function onUpClick(event : MouseEvent) : void
		{
			_slider.value -= _lineScrollSize;
			dispatchEvent(new Event(Event.CHANGE));
		}
		
		/**
		 * 
		 */
		override protected function onDownClick(event : MouseEvent) : void
		{
			_slider.value += _lineScrollSize;
			dispatchEvent(new Event(Event.CHANGE));
		}
		
		/**
		 * 
		 */
		protected function pageChangeHandler(event : Event) : void
		{
			if(event.type == BasicScrollSlider.PAGE_DOWN) _slider.value += _pageScrollSize;
			if(event.type == BasicScrollSlider.PAGE_UP) _slider.value -= _pageScrollSize;
			dispatchEvent(new Event(Event.CHANGE));
		}
	}
}
