package com.hisschemoller.view.components.ui 
{
	import com.bit101.components.Slider;
	import com.hisschemoller.view.components.enum.Colors;

	import flash.display.DisplayObjectContainer;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	/**
	 * @author Wouter Hisschemoller
	 * (c) Mar 20, 2010
	 */
	public class BasicSlider extends Slider 
	{
		public static const CLICK_TARGET_TRACK : String = "clickTargetTrack";
		public static const CLICK_TARGET_THUMB : String = "clickTargetThumb";
		protected var _back : Sprite;
		protected var _isDown : Boolean;

		public function BasicSlider(orientation : String = Slider.HORIZONTAL, parent : DisplayObjectContainer = null, xpos : Number = 0, ypos : Number = 0, defaultHandler : Function = null)
		{
			super(orientation, parent, xpos, ypos, defaultHandler);
			
			_tick = 0.00001;
			backClick = true;
		}

		/**
		 * Sets / gets the current value of this slider.
		 */
		override public function set value(value : Number) : void
		{
			if(!_isDown) doSetValue(value);
		}
		
		/**
		 * Gets / sets the maximum value of this slider.
		 */
		override public function set maximum(m:Number):void
		{
			_max = m;
			positionHandle();
		}
		
		/**
		 * Gets / sets the minimum value of this slider.
		 */
		override public function set minimum(m:Number):void
		{
			_min = m;
			positionHandle();
		}
		
		/**
		 * 
		 */
		public function get isDown() : Boolean
		{
			return _isDown;
		}

		/**
		 * Creates and adds the child display objects of this component.
		 */
		override protected function addChildren() : void
		{
			_back = new Sprite();
			addChild(_back);
			
			_handle = new Sprite();
			_handle.buttonMode = true;
			_handle.useHandCursor = true;
			_handle.addEventListener(MouseEvent.MOUSE_DOWN, onDrag);
			addChild(_handle);
		}

		/**
		 * Draws the back of the slider.
		 */
		override protected function drawBack() : void
		{
			_back.graphics.clear();
			_back.graphics.beginFill(Colors.WHITE);
			_back.graphics.drawRect(0, 0, _width, _height);
			_back.graphics.endFill();

			if(backClick)
			{
				_back.addEventListener(MouseEvent.MOUSE_DOWN, onBackClick);
			}
			else
			{
				_back.removeEventListener(MouseEvent.MOUSE_DOWN, onBackClick);
			}
		}

		/**
		 * Draws the handle of the slider.
		 */
		override protected function drawHandle() : void
		{	
			_handle.graphics.clear();
			_handle.graphics.beginFill(Colors.BLOOD_RED);
			
			if(_orientation == HORIZONTAL)
			{
				_handle.graphics.drawRect(0, 0, _height, _height);
			}
			else
			{
				_handle.graphics.drawRect(0, 0, _width, _width);
			}
			
			_handle.graphics.endFill();
			positionHandle();
		}

		/**
		 * @param position: Position of the mouse on the slider.
		 * @param clickTarget: Was _handle or _back clicked.
		 */
		protected function calculateValue(mousePosition : Number, clickTarget : String) : void
		{
			if(_orientation == Slider.HORIZONTAL) 
			{
				var range : Number = _back.width - _handle.width;
				var position : Number = (mousePosition * _back.scaleX) - (_handle.width * 0.5);
			} 
			else 
			{
				range = _back.height - _handle.height;
				position = range - (mousePosition * _back.scaleY) + (_handle.height * 0.5);
			}
			
			var newValue : Number = _min + ((position / range) * (_max - _min));
			
			doSetValue(newValue, clickTarget);
		}

		/**
		 * @param value
		 * @param clickTarget: _handle or _back.
		 */
		protected function doSetValue(value : Number, clickTarget : String = null) : void
		{
			var oldValue : Number = _value;
			
			if(_tick == 0)
			{
				_value = Math.max(_min, Math.min(value, _max));
			}
			else if(_tick == 1)
			{
				_value = Math.max(_min, Math.min(Math.round(value), _max));
			}
			else
			{
				var pow : Number = Math.pow(10, getPrecision(_tick));
				var snap : Number = _tick * pow;
				var rounded : Number = Math.round(value * pow);
				var snapped : Number = Math.round(rounded / snap) * snap;
				var val : Number = snapped / pow;
				_value = Math.max(_min, Math.min(val, _max));
			}
			
			/** Only dispatch if value has changed, dispatch when dragging */
			if(oldValue != _value && clickTarget != null)
			{
				dispatchEvent(new Event(Event.CHANGE));
			}
			
			positionHandle();
		}

		/**
		 * Get the number of decimals of a number. Used by set value().
		 */
		protected function getPrecision(num : Number) : Number
		{
			var s : String = num.toString();
			
			if (s.indexOf(".") == -1) return 0;
			
			return String(s.split(".").pop()).length;
		}

		/**
		 * Handler called when user clicks the background of the slider, causing the handle to move to that point. Only active if backClick is true.
		 * @param event The MouseEvent passed by the system.
		 */
		override protected function onBackClick(event : MouseEvent) : void
		{
			var position : Number = (_orientation == Slider.HORIZONTAL) ? _back.mouseX : _back.mouseY;
			calculateValue(position, CLICK_TARGET_TRACK);
			/** So the handle can be moved immediately on mouseDown. */
			onDrag(null);
		}

		/**
		 * Internal mouseDown handler. Starts dragging the handle.
		 * @param event The MouseEvent passed by the system.
		 */
		override protected function onDrag(event : MouseEvent) : void
		{
			_isDown = true;
			super.onDrag(event);
		}

		/**
		 * Internal mouseUp handler. Stops dragging the handle.
		 * @param event The MouseEvent passed by the system.
		 */
		override protected function onDrop(event : MouseEvent) : void
		{
			_isDown = false;
			super.onDrop(event);
		}
		
		/**
		 * Internal mouseMove handler for when the handle is being moved.
		 * @param event The MouseEvent passed by the system.
		 */
		override protected function onSlide(event:MouseEvent):void
		{
			var position : Number = (_orientation == Slider.HORIZONTAL) ? _back.mouseX : _back.mouseY;
			calculateValue(position, CLICK_TARGET_THUMB);
		}
	}
}
